@@ -10,16 +10,23 @@ The module provides the following main functions:
 
 """
 
+# load core module first
 from .properties import *
 from .helper import *
 from .io import *
 from .classify import *
 from .iterate import *
 from .analyse import *
+from .generate import *
+
+# load high level modules
+from .analyse_frontend import *
 from .analyse_triangles import *
 from .analyse_cooccurrence import *
 from .analyse_distribution import *
-from .generate import *
+from .analyse_simulate import *
 from .draw import *
+from .rbridge import *
 
-from .rbridge import *
\ No newline at end of file
+# create info database
+from .motif_infos import *
\ No newline at end of file
