@@ -52,7 +52,8 @@ def countMotifs(G : nx.Graph,
     :returns: number of occurences of the specified classes in either an array or a dict, cf. parameter array
 
     """
-    result = {key : 0 for key in classificator.names}
+    classes = classificator.info().classes
+    result = {key : 0 for key in classes}
     
     if processes == 0:
         classified = map(classificator, iterator)
@@ -75,7 +76,7 @@ def countMotifs(G : nx.Graph,
 
     # output format    
     if array:
-        return numpy.array([result[k] for k in classificator.names])
+        return numpy.array([result[k] for k in classes])
     
     return result
 
@@ -137,91 +138,6 @@ def count3SMotifs(G : nx.Graph, iterator = None, **kwargs):
         iterator = iterate3SMotifs(G)
     return count3Motifs(G, iterator, **kwargs)
 
-def countMultiMotifs(G : nx.Graph, 
-                     *arities, 
-                     roles = [], 
-                     iterator = None, 
-                     speedup : bool = False, 
-                     **kwargs):
-    """
-    Front-end function for counting multi-motifs. The default iterator is
-    :py:class:`sma.MultiMotifs`. As classificator serves :py:class:`sma.MultiMotifClassificator`.
-    All options of :py:meth:`sma.countMotifs` are available.
-    
-    **Speed up mode:** Due to its complexity, :py:class:`sma.MultiMotifClassificator`
-    is quite slow and cannot be pickled. This means that in cannot be used in 
-    multi-process mode. Setting the ``speedup`` parameter to ``True`` enables the
-    following mechanism: If the given set of arities is supported (see below), 
-    :py:class:`sma.MultiMotifClassificator` is replaced with a simpler classificator,
-    e.g. :py:class:`sma.ThreeMotifClassificator`. Likewise the motif iterator is 
-    replaced (set ``iterator`` to your favorite iterator to prevent this). In speed
-    up mode, one can set ``processes = 6`` to run the counting with 6 concurrent 
-    processes, say.
-    
-    Currently supported sets of arities (signatures):
-        
-        - ``1, 2``: see :py:class:`sma.ThreeMotifClassificator`
-        - ``2, 2``: see :py:class:`sma.FourMotifClassificator`
-        - ``1, 1, 1``: see see :py:class:`sma.Multi111Classificator`
-        - ``1, 1, 2``: see see :py:class:`sma.Multi121Classificator`
-        - ``1, 2, 2``: see see :py:class:`sma.Multi221Classificator`
-        - ``2, 2, 2``: see see :py:class:`sma.Multi222Classificator`
-    
-    In speed up mode execution times should be close to the execution times when
-    using functions like :py:meth:`sma.count4Motifs` directly. See the following
-    example done for a SEN with 22 vertices and 55 edges:
-        
-    .. code :: Python
-    
-        %timeit sma.count4Motifs(G)
-        # 100 loops, best of 3: 9.51 ms per loop
-        
-        %timeit sma.countMultiMotifs(madagascar, 2,2, roles=[1,0])
-        # 100 loops, best of 3: 11.6 ms per loop
-        
-        %timeit sma.countMultiMotifs(madagascar, 2,2, roles=[1,0], speedup = True)
-        # 100 loops, best of 3: 9.64 ms per loop
-    
-    See also :py:meth:`sma.countMultiMotifsSparse`.
-    
-    :param G: the graph
-    :param arities: arities for the motif source and the classificator
-    :param roles: roles of the level, cf. :py:class:`sma.MultiMotifClassificator`.
-    :param speedup: whether to use speed up mode, see above
-    :param iterator: a custom iterator, default is :py:class:`sma.MultiMotifs`.
-    :param kwargs: additional parameters for :py:meth:`sma.countMotifs`
-    """
-    if speedup:
-        signature = sma.classify._multiSignature(arities)
-        if signature == [1,2]:
-            positions = sma.classify._matchPositions([1,2], arities, roles)
-            if iterator is None:
-                iterator = iterate3Motifs(G, positions[0], positions[1])
-            return count3Motifs(G, iterator, **kwargs)
-        elif signature == [2,2]:
-            positions = sma.classify._matchPositions([2,2], arities, roles)
-            if iterator is None:
-                iterator = iterate4Motifs(G, positions[0], positions[1])
-            return count4Motifs(G, iterator, **kwargs)
-        multis = [
-                # signature, ordered,  classificator
-                ([1, 1, 1], [1, 1, 1], sma.Multi111MotifClassificator),
-                ([1, 1, 2], [1, 2, 1], sma.Multi121MotifClassificator),
-                ([1, 2, 2], [2, 2, 1], sma.Multi221MotifClassificator),
-                ([2, 2, 2], [2, 2, 2], sma.Multi222MotifClassificator),
-            ]
-        for sig, sig_ordered, classificator in multis:
-            if signature == sig:
-                positions = sma.classify._matchPositions(sig_ordered, arities, roles)
-                if iterator is None:
-                    iterator = sma.MultiMotifs(G, *sig_ordered, flatten = True, subsystems = positions)
-                return countMotifs(G, classificator(G), iterator, **kwargs)
-        print("Warning: Speed up for signature", signature, "failed, falling back to slow version.")
-    
-    if iterator is None:
-        iterator = sma.MultiMotifs(G, *arities)
-    return countMotifs(G, sma.MultiMotifClassificator(G, *arities, roles = roles), iterator, **kwargs)
-
 def count3MotifsSparse(G : nx.Graph, 
                        level0, 
                        level1, 
@@ -251,10 +167,10 @@ def count3MotifsSparse(G : nx.Graph,
           and the amount of motifs already encountered
     
     The penultimate step is based on the following observation: There are six different
-    types of 3-motifs. Let :math:`m \in \mathbb{N}^6` be a row vector denoting the numbers of
-    occurences of the motif types. Let :math:`C \in \mathbb{N}^{6 \\times 2}` denote 
+    classes of 3-motifs. Let :math:`m \in \mathbb{N}^6` be a row vector denoting the numbers of
+    occurences of the motif classes. Let :math:`C \in \mathbb{N}^{6 \\times 2}` denote 
     the matrix in :py:const:`sma.MOTIF3_EDGES` containing one row for every motif
-    type with two entries each. The first entry represents the number of ``level1``
+    class with two entries each. The first entry represents the number of ``level1``
     edged contained in this motif, the second the number of ``level0`` edges. Let
     :math:`V_i` denote the set of vertices in level :math:`i`, :math:`E_{i,j}` the 
     set of edges from level :math:`i` to level :math:`j`. Then we have the following
@@ -288,7 +204,7 @@ def count3MotifsSparse(G : nx.Graph,
     
     for edge in H.edges():
         v1, v2 = edge
-        if data[v1]['sesType'] ^ data[v2]['sesType']:
+        if data[v1]['sesType'] != data[v2]['sesType']:
             edges_count01 += 1
             e1 = v1 if data[v1]['sesType'] == level1 else v2
             s  = v1 if data[v1]['sesType'] == level0 else v2
@@ -302,11 +218,9 @@ def count3MotifsSparse(G : nx.Graph,
     result['II.C'] //= 2
 
     result_array = numpy.array([result[key] for key in sma.MOTIF3_NAMES])
-    counts = result_array @ sma.properties.MOTIF3_EDGES
+    counts = result_array @ sma.MOTIF3_EDGES
     V = sma.nodesCount(H)
     
-    # total = numpy.array([V[level0] * edges_count11, (V[level1]-1) * edges_count01])
-    
     result_array[3] = V[level0] * edges_count11 - counts[0]
     result_array[0] = V[level0] * V[level1] * (V[level1] - 1) // 2 - sum(result_array)
     
@@ -353,7 +267,7 @@ def count4MotifsSparse(G : nx.Graph,
     This means that motifs without cross-level edges (IV.A to IV.D) cannot be recognized.
     However the number of these motifs can be reconstructed based on the numbers
     for the other motifs. After completing the iteration we know the numbers of 
-    all motifs except for the types IV.A to IV.D. Based on the double counting 
+    all motifs except for the classes IV.A to IV.D. Based on the double counting 
     formulas outlined in :py:meth:`sma.count3MotifsSparse` we can compute the number
     of edges :math:`E_i` in level :math:`i` that we have not encountered yet. 
     The number of remaining motifs :math:`M` can be easily computed as well 
@@ -406,7 +320,7 @@ def count4MotifsSparse(G : nx.Graph,
     classificator = sma.FourMotifClassificator(G)
     for edge in H.edges():
         v1, v2 = edge
-        if data[v1]['sesType'] ^ data[v2]['sesType']:
+        if data[v1]['sesType'] != data[v2]['sesType']:
             edges_count01 += 1
             e1 = v1 if data[v1]['sesType'] == level1 else v2
             s1 = v1 if data[v1]['sesType'] == level0 else v2
@@ -465,8 +379,8 @@ def count121MotifsSparse(G : nx.Graph,
         - for each of them, iterating over all nodes from the middle level that
           are adjacent to the node in the lowest level
           
-    With default parameters, this function counts motifs of all types correctly.
-    If ``optimize_top_adjacent = True``, only types ``3`` and ``4`` are counted correctly.
+    With default parameters, this function counts motifs of all classes correctly.
+    If ``optimize_top_adjacent = True``, only classes ``3`` and ``4`` are counted correctly.
     
     :param G: the SEN
     :param level0: ``sesType`` of the upper level
@@ -474,13 +388,14 @@ def count121MotifsSparse(G : nx.Graph,
     :param level2: ``sesType`` of the lower level
     :param array: whether the output should be an array or a dict
     :param optimize_top_adjacent: whether the counting should be optimized for
-        counting motifs of types ``3`` and ``4``, see above.
+        counting motifs of classes ``3`` and ``4``, see above.
     :param debug_unclassified: whether, in case that ``optimize_top_adjacent = True``,
-        the partial count for type ``-1`` should be returned. Per default, the 
+        the partial count for class ``-1`` should be returned. Per default, the 
         value is set to a negative value.
     """
-    classificator = sma.Multi121MotifClassificator(G)
-    result = {k : 0 for k in classificator.names}
+    motif_info = sma.Motif121Info
+    classificator = motif_info.classificator(G)
+    result = {k : 0 for k in motif_info.classes}
     for lower in sma.sesSubgraph(G, level2):
         if optimize_top_adjacent:
             top = filter(lambda x: G.nodes[x]['sesType'] == level0, G[lower])
@@ -526,9 +441,9 @@ def count221MotifsSparse(G : nx.Graph,
     
     If ``optimize_top_adjacent = True``, this function iterates only over top level
     nodes which are adjacent to the lower level node. In this case only motifs of
-    types *CLASS*.3 are counted correctly. 
+    classes *CLASS*.3 are counted correctly. 
     
-    With default parameters, only motifs of types *CLASS*.2 and *CLASS*.3 are counted
+    With default parameters, only motifs of classes *CLASS*.2 and *CLASS*.3 are counted
     correctly. The values for ``Unclassified``, *CLASS*.1, *CLASS*.0 will be set 
     to a negative value.
     
@@ -542,14 +457,15 @@ def count221MotifsSparse(G : nx.Graph,
     :param level2: ``sesType`` of the lower level
     :param array: whether the output should be an array or a dict
     :param optimize_top_adjacent: whether the counting should be optimized for
-        counting motifs of types *CLASS*.3, see above.
+        counting motifs of classes *CLASS*.3, see above.
     :param processes: number of processes for multiprocessing
     :param chunksize: chunksize for multiprocessing
-    :param debug_unclassified: whether, the partial count for type ``Unclassified``
+    :param debug_unclassified: whether, the partial count for class ``Unclassified``
         should be returned. Per default, the value is set to a negative value.
     """
-    classificator = sma.Multi221MotifClassificator(G)
-    result = {k : 0 for k in classificator.names}
+    motif_info = sma.Motif221Info
+    classificator = motif_info.classificator(G)
+    result = {k : 0 for k in motif_info.classes}
     iterator = sma.DenseMulti221Motifs(G, level0, level1, level2, optimize_top_adjacent)
     
     if processes == 0:
@@ -568,9 +484,9 @@ def count221MotifsSparse(G : nx.Graph,
     # mark implausible results
     if not(debug_unclassified):
         result['Unclassified'] = -1
-    implausible = [2]
+    implausible = [0,1]
     if optimize_top_adjacent:
-        implausible = implausible + [0,1]
+        implausible = implausible + [2]
     for v in itertools.product(sma.MOTIF4_NAMES, implausible):
         result['{}.{}'.format(*v)] = -1
         
@@ -625,7 +541,7 @@ def count222MotifsSparse(G : nx.Graph,
           neighbours of both ends of the edge and over all top level nodes which
           are simulatenusly neighbours of both ends of the edge
     
-    This function counts only motifs of types ``3`` and ``4`` correclty. All other
+    This function counts only motifs of classes ``3`` and ``4`` correclty. All other
     counts are set to a negative value.
     
     This function supports multiprocessing. Relatively small chunksizes are 
@@ -638,7 +554,7 @@ def count222MotifsSparse(G : nx.Graph,
     :param array: whether the output should be an array or a dict
     :param processes: number of processes for multiprocessing
     :param chunksize: chunksize for multiprocessing
-    :param debug_unclassified: whether, the partial count for type ``-1``
+    :param debug_unclassified: whether, the partial count for class ``-1``
         should be returned. Per default, the value is set to a negative value.
     """
     result = numpy.zeros((3,), dtype=int)
@@ -664,50 +580,11 @@ def count222MotifsSparse(G : nx.Graph,
         return {k : v for k, v in zip([-1, 1, 2, 3, 4], real_result)}
     return numpy.array(real_result)
 
-def countMultiMotifsSparse(G : nx.Graph,
-                           *arities, 
-                           roles = [],
-                           **kwargs):
-    """
-    Front-end function for several counting functions for sparse networks. It supports
-    the same parameters for defining the motifs (arities, roles) as other multi-level
-    counting functions. Please check the documentations of the respective functions
-    as some of them might return partial counts only. The supported functions are:
-        
-        - :py:meth:`sma.count3MotifsSparse`
-        - :py:meth:`sma.count4MotifsSparse`
-        - :py:meth:`sma.count121MotifsSparse`
-        - :py:meth:`sma.count221MotifsSparse`
-        - :py:meth:`sma.count222MotifsSparse`
-    
-    See also :py:meth:`sma.countMultiMotifs`.
-    
-    :param G: the SEN
-    :param arities: arities of the motifs, cf. :py:class:`sma.MultiMotifClassificator`.
-    :param roles: roles of the levels, cf. :py:class:`sma.MultiMotifClassificator`.
-    :param kwargs: additional parameters for the counter functions, see above.
-    """
-    signature = sma.classify._multiSignature(arities)
-    multis = [
-                # signature, ordered,  function
-                ([1, 2],    [1, 2],    sma.count3MotifsSparse),
-                ([2, 2],    [2, 2],    sma.count4MotifsSparse),
-                ([1, 1, 2], [1, 2, 1], sma.count121MotifsSparse),
-                ([1, 2, 2], [2, 2, 1], sma.count221MotifsSparse),
-                ([2, 2, 2], [2, 2, 2], sma.count222MotifsSparse),
-            ]
-    for sig, sig_ordered, counter in multis:
-        if signature == sig:
-            positions = sma.classify._matchPositions(sig_ordered, arities, roles)
-            return counter(G, *positions, **kwargs)
-            
-    raise TypeError('No suitable sparse counter found')
-
 # Iterators
     
 def iterate4Motifs(G : nx.Graph, 
-                   level0 = sma.properties.NODE_TYPE_SOC, 
-                   level1 = sma.properties.NODE_TYPE_ECO):
+                   level0 = sma.NODE_TYPE_SOC, 
+                   level1 = sma.NODE_TYPE_ECO):
     """
     Yields all tuples (s1,s2,e1,e2) where s1,s2 are social nodes,
     e1,e2 are ecological nodes
@@ -742,8 +619,8 @@ def iterate3Motifs(G : nx.Graph, level0, level1):
                                  itertools.combinations(socialSubgraph, 2)))
 
 def iterate3EMotifs(G : nx.Graph,
-                    level0 = sma.properties.NODE_TYPE_ECO,
-                    level1 = sma.properties.NODE_TYPE_SOC):
+                    level0 = sma.NODE_TYPE_ECO,
+                    level1 = sma.NODE_TYPE_SOC):
     """
     Yields all tuples (e, s1, s2) where e is an ecological node and
     s1,s2 are social nodes.
@@ -760,8 +637,8 @@ def iterate3EMotifs(G : nx.Graph,
     return iterate3Motifs(G, level0, level1)
 
 def iterate3SMotifs(G : nx.Graph,
-                    level0 = sma.properties.NODE_TYPE_SOC,
-                    level1 = sma.properties.NODE_TYPE_ECO):
+                    level0 = sma.NODE_TYPE_SOC,
+                    level1 = sma.NODE_TYPE_ECO):
     """
     Yields all tuples (s, e1, e2) where s is a social node and 
     e1, e2 are ecological nodes.
@@ -885,17 +762,19 @@ def edgesCount(G : nx.Graph, array : bool =False) -> dict:
     def _classify(edge):
         v1, v2 = edge
         types = [G.nodes[v1]['sesType'], G.nodes[v2]['sesType']]
-        if types == [sma.properties.NODE_TYPE_SOC, sma.properties.NODE_TYPE_SOC]:
-            return sma.properties.EDGE_TYPE_SOC_SOC
-        if types == [sma.properties.NODE_TYPE_ECO, sma.properties.NODE_TYPE_ECO]:
-            return sma.properties.EDGE_TYPE_ECO_ECO
-        return sma.properties.EDGE_TYPE_ECO_SOC
+        if types == [sma.NODE_TYPE_SOC, sma.NODE_TYPE_SOC]:
+            return sma.EDGE_TYPE_SOC_SOC
+        if types == [sma.NODE_TYPE_ECO, sma.NODE_TYPE_ECO]:
+            return sma.EDGE_TYPE_ECO_ECO
+        if sma.NODE_TYPE_ECO in types and sma.NODE_TYPE_SOC in types:
+            return sma.EDGE_TYPE_ECO_SOC
+        raise TypeError('sma.edgesCount supports only two-level networks, use sma.edgesCountMatrix instead')
     classification = list(map(_classify, G.edges))
     types, counts = numpy.unique(classification, return_counts=True)
     if array:
-        return [int(counts[types == k][0]) if k in types else 0 for k in [sma.properties.EDGE_TYPE_SOC_SOC, sma.properties.EDGE_TYPE_ECO_SOC, sma.properties.EDGE_TYPE_ECO_ECO] ]
+        return [int(counts[types == k][0]) if k in types else 0 for k in [sma.EDGE_TYPE_SOC_SOC, sma.EDGE_TYPE_ECO_SOC, sma.EDGE_TYPE_ECO_ECO] ]
     else:
-        return { k : int(counts[types == k][0]) if k in types else 0 for k in [sma.properties.EDGE_TYPE_SOC_SOC, sma.properties.EDGE_TYPE_ECO_SOC, sma.properties.EDGE_TYPE_ECO_ECO] }
+        return { k : int(counts[types == k][0]) if k in types else 0 for k in [sma.EDGE_TYPE_SOC_SOC, sma.EDGE_TYPE_ECO_SOC, sma.EDGE_TYPE_ECO_ECO] }
 
 def edgesCountMatrix(G : nx.Graph, nTypes : int = None) -> numpy.ndarray:
     """
@@ -939,7 +818,7 @@ def adjacentEdgesCount(G : nx.Graph, node, nTypes = 2) -> numpy.ndarray:
     types, counts = numpy.unique(classification, return_counts = True)
     return numpy.array([counts[types == i][0] if i in types else 0 for i in range(nTypes)], dtype=int)
 
-def total4Motifs(G : nx.Graph, multi : bool = False) -> int:
+def total4Motifs(G : nx.Graph) -> int:
     """
     Returns the total number of 4-motifs in a graph, i.e.
     
@@ -949,20 +828,14 @@ def total4Motifs(G : nx.Graph, multi : bool = False) -> int:
         
     where n_s and n_e are respectively the numbers of social and ecological nodes.
     
-    You must switch the parameter ``multi`` on if you want to use this function
-    with multilevel networks, cf. :py:meth:`sma.totalMultiMotifs`.
+    See also :py:meth:`sma.totalMultiMotifs`.
     
     :param G: the graph
-    :param multi: whether the network is a multilevel network (default false)
+    :returns: total number of 4-motifs
     """
-    ecoNodes = len(list(sma.analyse.sesSubgraph(G, sma.properties.NODE_TYPE_ECO)))
-    if multi:
-        socNodes = len(list(sma.analyse.sesSubgraph(G, sma.properties.NODE_TYPE_SOC)))
-    else:
-        socNodes = len(G.nodes) - ecoNodes
-    return ecoNodes * (ecoNodes -1) * socNodes * (socNodes -1) // 4
+    return totalMultiMotifs(G, 2, 2)
 
-def total3EMotifs(G : nx.Graph, multi : bool = False) -> int:
+def total3EMotifs(G : nx.Graph) -> int:
     """
     Returns the total number of 3E-motifs in a graph, i.e.
     
@@ -972,20 +845,14 @@ def total3EMotifs(G : nx.Graph, multi : bool = False) -> int:
         
     where n_s and n_e are respectively the numbers of social and ecological nodes.
     
-    You must switch the parameter ``multi`` on if you want to use this function
-    with multilevel networks, cf. :py:meth:`sma.totalMultiMotifs`.
+    See also :py:meth:`sma.totalMultiMotifs`.
     
     :param G: the graph
-    :param multi: whether the network is a multilevel network (default false)
+    :returns: total number of 3E-motifs
     """
-    ecoNodes = len(list(sma.analyse.sesSubgraph(G, sma.properties.NODE_TYPE_ECO)))
-    if multi:
-        socNodes = len(list(sma.analyse.sesSubgraph(G, sma.properties.NODE_TYPE_SOC)))
-    else:
-        socNodes = G.number_of_nodes() - ecoNodes
-    return ecoNodes * socNodes * (socNodes -1) // 2
+    return totalMultiMotifs(G, 1, 2)
 
-def total3SMotifs(G : nx.Graph, multi : bool = False) -> int:
+def total3SMotifs(G : nx.Graph) -> int:
     """
     Returns the total number of 3S-motifs in a graph, i.e.
     
@@ -995,18 +862,12 @@ def total3SMotifs(G : nx.Graph, multi : bool = False) -> int:
         
     where n_s and n_e are respectively the numbers of social and ecological nodes.
     
-    You must switch the parameter ``multi`` on if you want to use this function
-    with multilevel networks, cf. :py:meth:`sma.totalMultiMotifs`.
+    See also :py:meth:`sma.totalMultiMotifs`.
     
     :param G: the graph
-    :param multi: whether the network is a multilevel network (default false)
+    :returns: total number of 3S-motifs
     """
-    ecoNodes = len(list(sma.analyse.sesSubgraph(G, sma.properties.NODE_TYPE_ECO)))
-    if multi:
-        socNodes = len(list(sma.analyse.sesSubgraph(G, sma.properties.NODE_TYPE_SOC)))
-    else:
-        socNodes = G.number_of_nodes() - ecoNodes
-    return socNodes * ecoNodes * (ecoNodes -1) // 2
+    return totalMultiMotifs(G, 2, 1)
 
 def totalMultiMotifs(G : nx.Graph, *arities : int) -> int:
     """
@@ -1034,10 +895,9 @@ def totalMultiMotifs(G : nx.Graph, *arities : int) -> int:
     """
     total = 1
     for level, arity in enumerate(arities):
-        if arity < 1:
-            continue
-        N = len(list(sma.analyse.sesSubgraph(G, level)))
-        total *= scipy.special.comb(N, arity, exact = True)
+        if arity >= 1:
+            N = len(list(sma.analyse.sesSubgraph(G, level)))
+            total *= scipy.special.comb(N, arity, exact = True)
     return total
 
 def sesTypes(G : nx.Graph) -> set:
