@@ -5,6 +5,7 @@ import networkx as nx
 import itertools, numpy
 import sma
 import multiprocessing
+import operator
 
 def cooccurrenceTable(G : nx.Graph, *motifs, to_array : bool = False, to_dict : bool = False):
     """
@@ -19,12 +20,12 @@ def cooccurrenceTable(G : nx.Graph, *motifs, to_array : bool = False, to_dict :
         import sma
         # let G be some SEN
         result = sma.cooccurrenceTable(G, 
-                                       sma.ThreeEMotifs(G) & sma.is3Type('I.C'), 
-                                       sma.ThreeEMotifs(G) & sma.is3Type('II.C'))
+                                       sma.ThreeEMotifs(G) & sma.is3Class('I.C'), 
+                                       sma.ThreeEMotifs(G) & sma.is3Class('II.C'))
     
     Then the first entry for each node contains the number of open triangles 
-    (type I.C motif) this node is involved in and the second entry the number of
-    closed triangles (type II.C motif). Note, that in this example the involvement
+    (class I.C motif) this node is involved in and the second entry the number of
+    closed triangles (class II.C motif). Note, that in this example the involvement
     of a node at any position (not only at the distinct position) counts, cf.
     :py:meth:`sma.triangleCoefficient`.
     
@@ -64,9 +65,9 @@ def cooccurrenceTableFull(G : nx.Graph,
     redundant costs.
     
     If the parameter ``to_array`` is set to ``False`` (default), the result is 
-    a dictionary which maps vertices to subdictionaries mapping motif types (taken
+    a dictionary which maps vertices to subdictionaries mapping motif classes (taken
     from the classificator) to the integer representing the number of occasions this
-    vertic occurs in a motif of this type. If ``to_array`` is flipped, the result 
+    vertic occurs in a motif of this class. If ``to_array`` is flipped, the result 
     is converted to a matrix with one row for each vertex and one column for each
     motif class.
     
@@ -78,7 +79,7 @@ def cooccurrenceTableFull(G : nx.Graph,
     :param classificator: a :py:class:`sma.MotifClassificator` for classifying the
         motifs
     :param to_array: whether the result should be an array or a dict.
-    :returns: cooccurrences table featuring values for all types of motifs
+    :returns: cooccurrences table featuring values for all classes of motifs
     
     """
     result = {node : {} for node in G.nodes}
@@ -90,7 +91,7 @@ def cooccurrenceTableFull(G : nx.Graph,
             else:
                 result[node][typ] = 1
     if to_array:
-        return numpy.array([[row[motif] if motif in row else 0 for motif in classificator.names] for row in result.values()])
+        return numpy.array([[row[motif] if motif in row else 0 for motif in classificator.info().classes] for row in result.values()])
     return result
     
 def motifMultigraph(G : nx.Graph, *motifs, attr = 'motif') -> nx.MultiGraph:
@@ -105,23 +106,23 @@ def motifMultigraph(G : nx.Graph, *motifs, attr = 'motif') -> nx.MultiGraph:
     shares trivially all its vertices with itself.
     
     A list of motif iterators must be given, cf. :py:class:`sma.MotifIterator`.
-    In this way different types of motifs can be incorporated in one multigraph.
+    In this way different classes of motifs can be incorporated in one multigraph.
     For example, one could be interested in the adjacence of open triangles with
     distinct social node (social motif I.C) and closed triangles with distinct 
     ecological node (ecological motif II.C). Then these two motif iterators can
     be provided as input, cf. :py:class:`sma.ThreeEMotifs`, :py:class:`sma.ThreeSMotifs`
-    and :py:class:`sma.is3Type`.
+    and :py:class:`sma.is3Class`.
     
     The multigraph can be converted to a weighted simple graph with edge weights
     corresponding to edge multiplicities in the multigraph using :py:meth:`sma.multiToWeightedGraph`.
     See also :py:meth:`sma.motifWeightedGraph`.
     
-    The type of the motifs, i.e. a string representation of the given motif 
+    The class of the motifs, i.e. a string representation of the given motif 
     iterators, is stored as nodal attribute.
     
     :param G: a SEN
     :param motifs: one or several instances of :py:class:`sma.MotifIterator`
-    :param attr: attribute key for the nodal attribute representing the type
+    :param attr: attribute key for the nodal attribute representing the class
         of the motifs, default ``motif``
     :returns: motif multigraph
     """
@@ -157,15 +158,15 @@ def motifClassMatrix(G : nx.Graph,
                      as_symmetric : bool = False) -> numpy.ndarray:
     """
     In the returned quadratic matrix every column and every row represents a motif
-    class, i.e. a type of motif as recorgnized by the given :py:class:`sma.MotifClassificator`.
+    class, i.e. a class of motif as recorgnized by the given :py:class:`sma.MotifClassificator`.
     For example, in case of 3-motifs the first column and the first row represent
-    type 'I.A' motifs whereas the last row / column represents 'II.C' motifs. In
+    class 'I.A' motifs whereas the last row / column represents 'II.C' motifs. In
     total, the matrix would be of dimension :math:`6 \\times 6` since there are six
     3-motifs.
     
     The :math:`(i,j)`-th entry represents the number of vertices in the given SEN shared
-    by motifs of type :math:`i` and :math:`j`. If a vertex is contained in several type 
-    :math:`i` and type :math:`j` motifs, it is counted multiply.
+    by motifs of class :math:`i` and :math:`j`. If a vertex is contained in several class 
+    :math:`i` and class :math:`j` motifs, it is counted multiply.
     
     Let :math:`(C_{ij})` denote the returned matrix. Then for the sum of all upper
     triangular entries the following correspondance holds:
@@ -199,7 +200,9 @@ def motifClassMatrix(G : nx.Graph,
     ctable = cooccurrenceTableFull(G, iterator, classificator, to_array = True)
     choose2 = lambda n : n*(n-1)//2
     diag = numpy.diag(sum(choose2(ctable)))
-    offdiag = numpy.array([[numpy.dot(ctable[:,i],ctable[:,j]) if i < j else 0 for j in range(classificator.classes)] for i in range(classificator.classes)])
+    offdiag = numpy.array([[numpy.dot(ctable[:,i],ctable[:,j]) if i < j else 0 
+                            for j in range(len(classificator.info().classes))] 
+                            for i in range(len(classificator.info().classes))])
     if as_symmetric:
         return offdiag + diag + numpy.transpose(offdiag)
     else:
@@ -219,25 +222,31 @@ def motifClassGraph(G : nx.Graph,
         the motifs. Note that it must match with the given iterator.
     """
     graph = nx.from_numpy_array(motifClassMatrix(G, iterator, classificator, as_symmetric=True))
-    return nx.relabel_nodes(graph, dict(zip(itertools.count(), classificator.names)), copy=True)
+    return nx.relabel_nodes(graph, dict(zip(itertools.count(), classificator.info().classes)), copy=True)
 
 class _cooccurrenceEdgeTableFullMapper:
     def __init__(self, classificator, nodes_rows, nodes_columns, dyad):
         self.classificator = classificator
         self.indexer_rows    = {n : i for n, i in zip(nodes_rows, itertools.count())}
         self.indexer_columns = {n : i for n, i in zip(nodes_columns, itertools.count())}
-        self.indexer_motifs  = {n : i for n, i in zip(classificator.names, itertools.count())}
+        self.indexer_motifs  = {n : i for n, i in zip(classificator.info().classes, itertools.count())}
         self.dyad = dyad
     def __call__(self, motif):
         cl = self.classificator(motif)
-        return (self.indexer_rows[motif[self.dyad[0]]], 
-                self.indexer_columns[motif[self.dyad[1]]], 
-                self.indexer_motifs[cl])
+        if type(self.dyad) is int:
+            return (self.indexer_rows[motif[self.dyad][0]], 
+                    self.indexer_columns[motif[self.dyad][1]], 
+                    self.indexer_motifs[cl]) 
+        else:
+            return (self.indexer_rows[motif[self.dyad[0]]], 
+                    self.indexer_columns[motif[self.dyad[1]]], 
+                    self.indexer_motifs[cl])
+
 
 def cooccurrenceEdgeTableFull(G : nx.Graph,
                               iterator : sma.MotifIterator,
                               classificator : sma.MotifClassificator,
-                              dyad : tuple,
+                              dyad,
                               levels : tuple,
                               processes : int = 0,
                               chunksize : int = 10000):
@@ -248,7 +257,7 @@ def cooccurrenceEdgeTableFull(G : nx.Graph,
     :math:`V_j` denotes the set of notes from level :math:`i`, resp. :math:`j` in
     the SEN and :math:`M` denotes the number of motif classes as classified by the
     given classificator. The :math:`(a,b,c)`th entry contains the number of times
-    the edge :math:`(a,b)` occurs in a motif of type :math:`c`.
+    the edge :math:`(a,b)` occurs in a motif of class :math:`c`.
     
     The edge is specified using the ``dyad`` parameter. This parameter must contain
     a tuple of two integers specifying the index of the nodes spanning the edge in 
@@ -289,8 +298,11 @@ def cooccurrenceEdgeTableFull(G : nx.Graph,
     else:
         nodes_columns = list(sma.sesSubgraph(G, levels[0]))
     
-    matrix = numpy.zeros((len(nodes_rows), len(nodes_columns), classificator.classes), dtype=int)
-    mapper = _cooccurrenceEdgeTableFullMapper(classificator, nodes_rows, nodes_columns, dyad)
+    matrix = numpy.zeros((len(nodes_rows), len(nodes_columns), len(classificator.info().classes)), dtype=int)
+    mapper = _cooccurrenceEdgeTableFullMapper(classificator, 
+                                              nodes_rows, 
+                                              nodes_columns, 
+                                              dyad)
     
     if processes == 0:
         for index in map(mapper, iterator):
@@ -306,7 +318,123 @@ def cooccurrenceEdgeTableFull(G : nx.Graph,
             p.join()
     
     if levels[0] == levels[1]:
-        for i in range(classificator.classes):
+        for i in range(len(classificator.info().classes)):
             matrix[:,:,i] = numpy.triu(matrix[:,:,i] + matrix[:,:,i].T) - numpy.diag(numpy.diag(matrix[:,:,i]))
     
     return matrix, nodes_rows, nodes_columns
+
+def edgeContributionList(G : nx.Graph, *arities, roles = [], level : int) -> numpy.ndarray:
+    """
+    Computes edge contribution lists for a certain family of motifs. This list
+    contains one entry for every possible edge in the level of the network specified
+    by ``level``. The entry associated with a dyad is number of times this dyad
+    occurs in a certain family of motifs. The dyad itself is not taken into account
+    here. In this way, open and closed motifs (i.e. motifs with a dyad in level
+    ``level`` and otherwise equivalent motifs without this dyad) are grouped together.
+    The value measures the contribution of a dyad in level ``level`` to open/closed
+    motifs of similar class.
+    
+    This only makes sense if the motif specified by ``arities`` (and ``roles``)
+    contains two nodes in level ``level``.
+    
+    For example, when working with 3-motifs (and ``level`` equalling the ``sesType``
+    of the non-distinct nodes), motifs I.A and II.A, I.B and II.B and I.C and II.C
+    are grouped together.
+    
+    :param G: the SEN
+    :param arities: arities of the (multi-level) motif
+    :param roles: optional roles for position matching, cf. :py:meth:`sma.matchPositions`
+    :param level: level as described above
+    :returns: tuple of:
+        
+        - two-dimensional array with one row for every group of motif (grouped
+          by open/closed) and one column for every dyad on level ``level``
+        - iterator giving the edges which index the columns of the array, call 
+          ``list()`` on it to obtain a list of indices
+    
+    :raises NotImplementedError: if the motif signature is not supported
+    """
+    signature  = sma.multiSignature(arities)
+    motif_info = sma.motifInfo(signature)
+    positions  = sma.matchPositions(motif_info.signature, arities, roles)
+    assert level in positions, 'selected motif does not reach selected level'
+    index = positions.index(level)
+    assert motif_info.signature[index] == 2, 'motif does not contain 2 vertices on the requested level'
+    offset = sum(motif_info.signature[:index])
+
+    iterator   = sma.MultiMotifsNormalized(G, *arities, roles = roles, motif_info = motif_info)
+    classificator = motif_info.classificator(G)
+    
+    table, nodes, _ = sma.cooccurrenceEdgeTableFull(G,
+                                                    iterator,
+                                                    classificator,
+                                                    (offset, offset+1),
+                                                    (level, level))
+
+    per_motif = numpy.array([table[:,:,i][numpy.triu_indices(table.shape[0],1)] for i in range(table.shape[2])])
+    
+    # group together
+    indexer = {motif : i for i, motif in enumerate(motif_info.classes)}
+    abstract_level = positions.index(level)
+    if abstract_level not in motif_info.projections:
+        raise NotImplementedError('ActorsChoice not implemented for signature %s on level %d' \
+                                  % (str(motif_info.signature), level))
+    grouper = motif_info.projections[abstract_level]
+    result  = numpy.array([sum(per_motif[indexer[gr]] for gr in group) for group in grouper])
+    
+    edges = itertools.combinations(nodes, 2)
+    return result, edges
+
+def identifyGaps(G : nx.Graph, 
+                 motif_identifier : str, 
+                 level : int = -1) -> list:
+    """
+    Computes a list of gaps (specific dyads) together with their contribution to 
+    the number of motifs of a specific class.
+    
+    The contribution of a gap is the number of motifs of a fixed class which would
+    be created by replacing the gap with an edge. This function computes this contribution
+    for every gap on the specified level.
+    
+    The behaviour above applies when the specified motif contains the edge on the
+    given level (i.e. the motif is a *closed motif*). If the edge is not present
+    in the specified motif (the motif is *open*), the contribution is definied the
+    number of motifs of the specified class are created by removing an edge from the
+    network.
+    
+    :param G: the SEN
+    :param motif_identifier: motif identifier string of the motif
+    :param level: dyads of which level (``sesType``) are to be considered
+    :returns: list of tuples ordered by the second entry in descending order
+        consisting of
+    
+        - a tuple specifying an edge
+        - the contribution of this edge to the number of motifs of the given type
+          when being flipped
+    
+    """
+    arities, roles, motif = sma.parseMotifIdentifier(motif_identifier)
+    signature  = sma.multiSignature(arities)
+    motif_info = sma.motifInfo(signature)
+    if level < 0:
+        assert 2 in motif_info.signature, 'signature does not contain two'
+        positions  = sma.matchPositions(motif_info.signature, arities, roles)
+        level = positions[signature.index(2)]
+    contributions, edges = sma.edgeContributionList(G, 
+                                                    *motif_info.signature, 
+                                                    roles = positions, 
+                                                    level = level)
+    abstract_level = positions.index(level)
+    grouper = motif_info.projections[abstract_level]
+    index, group = next(filter(lambda x: motif in x[1], enumerate(grouper)))
+    # we want to identify gaps, so ties which would close the largest number of gaps,
+    # i.e. whose presence would increase the number of motifs specified by motif_idenitifer
+    row = contributions[index]
+    values = zip(edges, row)
+    if group[0] == motif:
+        # motif is an open motif
+        values_filtered = filter(lambda x : G.has_edge(*x[0]), values)
+    else:
+        # motif is a closed motif
+        values_filtered = filter(lambda x : not(G.has_edge(*x[0])), values)
+    return sorted(values_filtered, key=operator.itemgetter(1), reverse=True)
\ No newline at end of file
